@@ -1,7 +1,26 @@
+/*
+ *  mugnet : Mixed Categorical Bayesian networks
+ *  Copyright (C) 2009--2010  Nikolay Balov
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.gnu.org/licenses/gpl-2.0.html
+ */
+
 /*
  * mixnet.cpp
  *
- *  Created on: Nov 16, 2009
+ *  Created on: Nov 11, 2010
  *      Author: Nikolay Balov
  */
 
@@ -112,6 +131,11 @@ int CMixNet::pcC_size() {
 void CMixNet::_release() {
 	int i;
 
+	if(m_pCatnetSamples)
+		CATNET_FREE(m_pCatnetSamples);
+	m_pCatnetSamples = 0; 
+	m_nCatnetSamples = 0;
+
 	if(m_betas) { 
 		for (i = 0; i < m_numNodes; i++) {
 			if(m_betas[i])
@@ -148,10 +172,6 @@ void CMixNet::release_pqcC() {
 	if(m_qcC)
 		CATNET_FREE(m_qcC);
 	m_qcC = 0;
-	if(m_pCatnetSamples)
-		CATNET_FREE(m_pCatnetSamples);
-	m_pCatnetSamples = 0; 
-	m_nCatnetSamples = 0;
 }
 
 void CMixNet::_reset() {
@@ -225,6 +245,49 @@ double CMixNet::loglik() {
 	return m_loglik;
 }
 
+double CMixNet::estimateParameters(int nnode, double *psamples, int nsamples, 
+					double *pBetas, double *pSigma) {
+	double ftemp, fsum, fBetaSigmaLoglik;
+	int ic, j, c_setSize;
+
+	if(m_curNode != nnode)
+		return -FLT_MAX;
+
+	c_setSize = m_numCategories[nnode];
+	for(ic = 0; ic < c_setSize; ic++) {
+		ftemp = 0;
+		fsum = 0;
+		for(j = 0; j < nsamples; j++) {
+			ftemp += m_qc[j*c_setSize + ic] * psamples[j * m_numNodes + nnode]; 
+			fsum += m_qc[j*c_setSize + ic];
+		}
+		if(fsum > 0) {
+			/* update only if possible, don't put 0, remember the last good value instead */
+			pBetas[ic] = ftemp / fsum;
+		}
+		//printf("fsum = %f, pBetasNext[%d][%d] = %f\n", fsum, nnode, ic, pBetasNext[nnode][ic]);
+	}
+
+	fsum = 0;
+	for(ic = 0; ic < c_setSize; ic++) {
+		for(j = 0; j < nsamples; j++) {
+			ftemp = psamples[j * m_numNodes + nnode] - pBetas[ic];
+			ftemp = ftemp * ftemp;
+			fsum += m_qc[j*c_setSize + ic] * ftemp;
+		}
+	}
+	*pSigma = 0;
+	if(fsum > 0) 
+		*pSigma = fsum / nsamples;
+	//else printf("fsum == 0\n");
+	if(*pSigma > 0)
+		fBetaSigmaLoglik = -0.5*(log(PI2**pSigma) + 1)*nsamples;
+	else 
+		fBetaSigmaLoglik = FLT_MAX;
+	//printf("node = %d, fsum = %f, fBetaSigmaLoglik = %f, sigma = %f\n", nnode, fsum, fBetaSigmaLoglik, *pSigma);
+	return fBetaSigmaLoglik;
+}
+
 int CMixNet::findNodeMarginalProb(int nnode, double *psamples, int nsamples) {
 
 	int res, j, ic, numcats;
@@ -243,9 +306,9 @@ int CMixNet::findNodeMarginalProb(int nnode, double *psamples, int nsamples) {
 
 	m_curNode = nnode;
 
-	sigma2 = 0;
+	sigma2 = FLT_MAX;
 	if(m_sigmas[nnode] > 0)
-		sigma2 =  - 1 / (2*m_sigmas[nnode]);
+		sigma2 =  1 / (2*m_sigmas[nnode]);
 	numcats = m_numCategories[nnode];
 
 	m_pc = (double*)CATNET_MALLOC(numcats*sizeof(double));
@@ -274,11 +337,13 @@ int CMixNet::findNodeMarginalProb(int nnode, double *psamples, int nsamples) {
 		fsum = 0;
 		for(ic = 0; ic < numcats; ic++) {
 			fdiff = psamples[j * m_numNodes + nnode] - m_betas[nnode][ic];
-			paux[ic] = m_pc[ic]*exp(sigma2*fdiff*fdiff); 
+			paux[ic] = m_pc[ic]*exp(-sigma2*fdiff*fdiff); 
 			fsum += paux[ic];
 		}
 		if(fsum > 0) 
 			fsum = 1/fsum;
+		else
+			fsum = FLT_MAX;
 		for(ic = 0; ic < numcats; ic++) {
 			m_qc[j*numcats + ic] = fsum*paux[ic];
 		}
@@ -292,7 +357,7 @@ int CMixNet::findNodeMarginalProb(int nnode, double *psamples, int nsamples) {
 
 int CMixNet::findNodeJointProb(int nnode, int *parnodes, int numpars, double *psamples, int nsamples) {
 
-	int res, i, j, ic, icc, k;
+	int res, i, j, ic, icc;
 	int *pnodes, numnodes, *pcats, numcats;
 	double *psigma2, fdiff, fdiffsum, fsum, *paux;
 
@@ -338,10 +403,12 @@ int CMixNet::findNodeJointProb(int nnode, int *parnodes, int numpars, double *ps
 	for(i = 0; i < numnodes; i++) {
 		if(m_sigmas[pnodes[i]] > 0)
 			psigma2[i] = 1 / (2*m_sigmas[pnodes[i]]);
+		else
+			psigma2[i] = FLT_MAX;
 	}
 
 	paux = (double*)CATNET_MALLOC(m_parCatSetSize*sizeof(double));
-	pcats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
+	pcats = (int*)CATNET_MALLOC(m_parCatSetSize*numnodes*sizeof(int));
 	m_pcC = (double*)CATNET_MALLOC(m_parCatSetSize*sizeof(double));
 	m_qcC = (double*)CATNET_MALLOC(m_parCatSetSize*nsamples*sizeof(double));
 
@@ -363,20 +430,27 @@ int CMixNet::findNodeJointProb(int nnode, int *parnodes, int numpars, double *ps
 			// determine the category indices
 			fsum = ic;
 			for(i = 0; i < numnodes; i++) {
-				pcats[i] = (int)(fsum/m_pBlockSizes[i]);
-				fsum -= pcats[i] * m_pBlockSizes[i];
+				pcats[ic * numnodes + i] = (int)(fsum/m_pBlockSizes[i]);
+				fsum -= pcats[ic * numnodes + i] * m_pBlockSizes[i];
 			}
-			m_pcC[ic] = getCatProb(pcats, numnodes);
+			m_pcC[ic] = getCatProb(pcats + ic * numnodes, numnodes);
 		}
 	}
 	else {
+		if(!m_pBlockSizes || m_nBlockSizes < numpars) {
+			if(m_pBlockSizes)
+				CATNET_FREE(m_pBlockSizes);
+			m_nBlockSizes = numpars;
+			m_pBlockSizes = (int*) CATNET_MALLOC(m_nBlockSizes * sizeof(int));
+		}
+
 		m_pBlockSizes[numnodes - 1] = 1;
 		for (i = numnodes - 2; i >= 0; i--) {
 			m_pBlockSizes[i] = m_pBlockSizes[i + 1] * m_numCategories[pnodes[i + 1]];
 			
 		}
 
-		memset(m_pcC, 0, m_nCatnetSamples*sizeof(double));
+		memset(m_pcC, 0, m_parCatSetSize*sizeof(double));
 
 		for(j = 0; j < m_nCatnetSamples; j++) {
 			ic = 0;
@@ -388,31 +462,36 @@ int CMixNet::findNodeJointProb(int nnode, int *parnodes, int numpars, double *ps
 		fsum = 1/(double)m_nCatnetSamples;
 		for(ic = 0; ic < m_parCatSetSize; ic++)
 			m_pcC[ic] *= fsum;
+
+		for(ic = 0; ic < m_parCatSetSize; ic++) {
+			// determine the category indices
+			fsum = ic;
+			for(i = 0; i < numnodes; i++) {
+				pcats[ic * numnodes + i] = (int)(fsum/m_pBlockSizes[i]);
+				fsum -= pcats[ic * numnodes + i] * m_pBlockSizes[i];
+			}
+		}
 	}
 
 	for (j = 0; j < nsamples; j++) {
 		fsum = 0;
 		for(ic = 0; ic < m_parCatSetSize; ic++) {
-			// determine the category indices
-			k = ic;
-			for(i = 0; i < numnodes; i++) {
-				pcats[i] = (int)(k/m_pBlockSizes[i]);
-				k -= pcats[i] * m_pBlockSizes[i];
-			}
-
 			// a blunder: only one beta per nnode
 			// would be nice to have m_betas[nnode][pcats[ic]]
 			//fdiff = psamples[j * m_numNodes + nnode] - m_betas[nnode][pcats[numnodes-1]];
 			// The way to GO
 			fdiffsum = 0;
 			for(i = 0; i < numnodes; i++) {
-				fdiff = psamples[j * m_numNodes + pnodes[i]] - m_betas[pnodes[i]][pcats[i]];
+				fdiff = psamples[j * m_numNodes + pnodes[i]] - m_betas[pnodes[i]][pcats[ic * numnodes + i]];
 				fdiffsum += psigma2[i]*fdiff*fdiff;
 			}
 			paux[ic] = m_pcC[ic]*exp(-fdiffsum);
 			fsum += paux[ic];
 		}
-		if(fsum > 0) fsum = 1/fsum;
+		if(fsum > 0) 
+			fsum = 1/fsum;
+		else
+			fsum = FLT_MAX;
 		for(ic = 0; ic < m_parCatSetSize; ic++)
 			m_qcC[j*m_parCatSetSize + ic] = fsum*paux[ic];
 	}
@@ -446,7 +525,7 @@ int CMixNet::findNodeJointProb(int nnode, int *parnodes, int numpars, double *ps
 
 int CMixNet::findParentsJointProb(int *parnodes, int numpars, double *psamples, int nsamples) {
 
-	int res, i, j, ic, k;
+	int res, i, j, ic;
 	int *pcats;
 	double *psigma2, fdiff, fdiffsum, fsum, *paux;
 
@@ -455,11 +534,6 @@ int CMixNet::findParentsJointProb(int *parnodes, int numpars, double *psamples,
 	if(!parnodes || !numpars)
 		return ERR_CATNET_PARAM;
 
-	res = marginalProb(parnodes, numpars);
-	if(res != ERR_CATNET_OK) {
-		return res;
-	}
-
 	if(m_pcC) CATNET_FREE(m_pcC);
 	m_pcC = 0;
 	if(m_qcC) CATNET_FREE(m_qcC);
@@ -469,11 +543,68 @@ int CMixNet::findParentsJointProb(int *parnodes, int numpars, double *psamples,
 	m_parCatSetSize = 1;
 	for(i = 0; i < numpars; i++)
 		m_parCatSetSize *= m_numCategories[parnodes[i]];
-	//m_parCatSet = (int*)CATNET_MALLOC(m_parCatSetSize*sizeof(int));
 
-	if(m_margProbSize != m_parCatSetSize) {
-		// fatal  error
-		return ERR_CATNET_PROC;
+	m_pcC = (double*)CATNET_MALLOC(m_parCatSetSize*sizeof(double));
+	pcats = (int*)CATNET_MALLOC(m_parCatSetSize*numpars*sizeof(int));
+	if(!m_pcC || !pcats)
+		return ERR_CATNET_MEM;
+	memset(m_pcC, 0, m_parCatSetSize*sizeof(double));
+
+	if(!m_pCatnetSamples || m_nCatnetSamples < 1) {
+
+		res = marginalProb(parnodes, numpars);
+		if(res != ERR_CATNET_OK) {
+			CATNET_FREE(pcats);
+			return res;
+		}
+
+		if(m_margProbSize != m_parCatSetSize) {
+			CATNET_FREE(pcats);
+			return ERR_CATNET_PROC;
+		}
+
+		for(ic = 0; ic < m_parCatSetSize; ic++) {
+			// determine the category indices
+			fsum = ic;
+			for(i = 0; i < numpars; i++) {
+				pcats[ic * numpars + i] = (int)(fsum/m_pBlockSizes[i]);
+				fsum -= pcats[ic * numpars + i] * m_pBlockSizes[i];
+			}
+			m_pcC[ic] = getCatProb(pcats + ic * numpars, numpars);
+		}
+	}
+	else {
+		if(!m_pBlockSizes || m_nBlockSizes < numpars) {
+			if(m_pBlockSizes)
+				CATNET_FREE(m_pBlockSizes);
+			m_nBlockSizes = numpars;
+			m_pBlockSizes = (int*) CATNET_MALLOC(m_nBlockSizes * sizeof(int));
+		}
+
+		m_pBlockSizes[numpars - 1] = 1;
+		for (i = numpars - 2; i >= 0; i--) {
+			m_pBlockSizes[i] = m_pBlockSizes[i + 1] * m_numCategories[parnodes[i + 1]];
+			
+		}
+
+		for(j = 0; j < m_nCatnetSamples; j++) {
+			ic = 0;
+			for (i = 0; i < numpars; i++)
+				ic += (m_pBlockSizes[i] * m_pCatnetSamples[j * m_numNodes + parnodes[i]]);
+			m_pcC[ic] += 1;
+		}
+		fsum = 1/(double)m_nCatnetSamples;
+		for(ic = 0; ic < m_parCatSetSize; ic++)
+			m_pcC[ic] *= fsum;
+
+		for(ic = 0; ic < m_parCatSetSize; ic++) {
+			// determine the category indices
+			fsum = ic;
+			for(i = 0; i < numpars; i++) {
+				pcats[ic * numpars + i] = (int)(fsum/m_pBlockSizes[i]);
+				fsum -= pcats[ic * numpars + i] * m_pBlockSizes[i];
+			}
+		}
 	}
 
 	psigma2 = (double*)CATNET_MALLOC(numpars*sizeof(double));
@@ -481,41 +612,30 @@ int CMixNet::findParentsJointProb(int *parnodes, int numpars, double *psamples,
 	for(i = 0; i < numpars; i++) {
 		if(m_sigmas[parnodes[i]] > 0)
 			psigma2[i] = 1 / (2*m_sigmas[parnodes[i]]);
+		else
+			psigma2[i] = FLT_MAX;
 	}
 
 	paux = (double*)CATNET_MALLOC(m_parCatSetSize*sizeof(double));
-	pcats = (int*)CATNET_MALLOC(numpars*sizeof(int));
-
-	m_pcC = (double*)CATNET_MALLOC(m_parCatSetSize*sizeof(double));
-	for(ic = 0; ic < m_parCatSetSize; ic++) {
-		// determine the category indices
-		fsum = ic;
-		for(i = 0; i < numpars; i++) {
-			pcats[i] = (int)(fsum/m_pBlockSizes[i]);
-			fsum -= pcats[i] * m_pBlockSizes[i];
-		}
-		m_pcC[ic] = getCatProb(pcats, numpars);
-	}
-
 	m_qcC = (double*)CATNET_MALLOC(m_parCatSetSize*nsamples*sizeof(double));
+	if(!m_qcC || !paux)
+		return ERR_CATNET_MEM;
+
 	for (j = 0; j < nsamples; j++) {
 		fsum = 0;
 		for(ic = 0; ic < m_parCatSetSize; ic++) {
-			// determine the category indices
-			k = ic;
-			for(i = 0; i < numpars; i++) {
-				pcats[i] = (int)(k/m_pBlockSizes[i]);
-				k -= pcats[i] * m_pBlockSizes[i];
-			}
 			fdiffsum = 0;
 			for(i = 0; i < numpars; i++) {
-				fdiff = psamples[j * m_numNodes + parnodes[i]] - m_betas[parnodes[i]][pcats[i]];
+				fdiff = psamples[j * m_numNodes + parnodes[i]] - m_betas[parnodes[i]][pcats[ic * numpars + i]];
 				fdiffsum += psigma2[i]*fdiff*fdiff;
 			}
 			paux[ic] = m_pcC[ic]*exp(-fdiffsum);
 			fsum += paux[ic];
 		}
-		if(fsum > 0) fsum = 1/fsum;
+		if(fsum > 0) 
+			fsum = 1/fsum;
+		else
+			fsum = FLT_MAX;
 		for(ic = 0; ic < m_parCatSetSize; ic++)
 			m_qcC[j*m_parCatSetSize + ic] = fsum*paux[ic];
 	}
@@ -537,8 +657,10 @@ int* CMixNet::catnetSample(int nsamples) {
 	if(nsamples < 1 || m_numNodes < 1)
 		return 0;
 	porder = getOrder();
-	if(!porder) 
+	if(!porder) {
+		warning("porder=0");
 		return 0;
+	}
 
 	if(m_pCatnetSamples)
 		CATNET_FREE(m_pCatnetSamples);
@@ -641,7 +763,7 @@ int CMixNet::sample(double *psamples, int nsamples) {
 int CMixNet::predict(double *psamples, int nsamples) {
 
 	int *porder;
-	int j, k, ic, icC, nnode, imax, cC_setSize;
+	int j, k, ic, icC, nnode, cC_setSize;
 	double fsum, faux, *pnodeprob;
 	PROB_LIST<double>* pProbList;
 
@@ -682,7 +804,6 @@ int CMixNet::predict(double *psamples, int nsamples) {
 			// predict only NA ones
 			if(!isnan(psamples[j*m_numNodes + nnode]))
 				continue;
-if(1) {
 			fsum = 0;
 			for(ic = 0; ic < m_numCategories[nnode]; ic++) {
 				faux = 0;
@@ -693,23 +814,6 @@ if(1) {
 				fsum += m_betas[nnode][ic] * faux;
 			}
 			psamples[j*m_numNodes + nnode] = fsum;
-}
-else {
-			imax = 0;
-			fsum = -FLT_MAX;
-			for(ic = 0; ic < m_numCategories[nnode]; ic++) {
-				faux = 0;
-				for(icC = 0; icC < cC_setSize; icC++) {
-					faux += pnodeprob[ic*m_numCategories[nnode] + ic] * 
-						m_qcC[j*cC_setSize + icC];
-				}
-				if(fsum < faux) {
-					fsum = faux;
-					imax = ic;
-				}
-			}
-			psamples[j*m_numNodes + nnode] = m_betas[nnode][imax];
-}
 		}
 	}
 
@@ -718,13 +822,13 @@ else {
 	return ERR_CATNET_OK;
 }
 
-double CMixNet::findNodeLoglik(int nnode, double *psamples, int nsamples) {
+double CMixNet::findLogNodeLikelihood(int nnode, double *psamples, int nsamples) {
 	
 	double faux, fsum, fsumsum, fLogLik, fconst1, fconst2;
 	int j, cC_setSize, ic, icC;
 	double *pnodeprob;
 	PROB_LIST<double>* pProbList;
-
+	
 	if(!m_sigmas || !m_betas || nnode < 0 || nnode >= m_numNodes || m_sigmas[nnode] <= 0)
 		return -FLT_MAX;
 	
@@ -734,6 +838,12 @@ double CMixNet::findNodeLoglik(int nnode, double *psamples, int nsamples) {
 	pProbList = (PROB_LIST<double>*)getNodeProb(nnode);
 	pnodeprob = pProbList->pProbs;
 
+	//printf("prob %d: ", nnode);
+	//for(ic = 0; ic < pProbList->nProbSize; ic++) {
+		//printf(" %f ", pnodeprob[ic]);
+	//}
+	//printf("\n");
+			
 	cC_setSize = 0;
 	if(m_numParents[nnode] > 0) {
 		if(findParentsJointProb(m_parents[nnode], m_numParents[nnode], psamples, nsamples) != ERR_CATNET_OK)
@@ -742,7 +852,7 @@ double CMixNet::findNodeLoglik(int nnode, double *psamples, int nsamples) {
 		cC_setSize = pcC_size();
 	}
 
-	fconst1 = log(1 / sqrt(PI2*m_sigmas[nnode]));
+	fconst1 = -0.5 * log(PI2*m_sigmas[nnode]);
 	fconst2 = -0.5 / m_sigmas[nnode];
 
 	fLogLik = 0;
@@ -756,15 +866,84 @@ double CMixNet::findNodeLoglik(int nnode, double *psamples, int nsamples) {
 				for(icC = 0; icC < cC_setSize; icC++) {
 					fsum += pnodeprob[icC*m_numCategories[nnode] + ic] * 
 						m_qcC[j*cC_setSize + icC];
+					//printf("      %f, %f\n", pnodeprob[icC*m_numCategories[nnode] + ic], m_qcC[j*cC_setSize + icC]);
 				}
 			}
 			else
 				fsum = pnodeprob[ic];
+			//printf("faux = %f, fsum = %f\n", faux, fsum);
 			fsumsum += exp(faux) * fsum;
 		}
 		fLogLik += log(fsumsum);
 	}
 	fLogLik += nsamples*fconst1;
+	//printf("nnode = %d:  fLogLik = %f\n", nnode, fLogLik);
+	return fLogLik;
+}
+
+
+double CMixNet::findNodeLoglik(int nnode, double *psamples, int nsamples) {
+	
+	double faux, fLogLik, fconst1, fconst2;
+	int j, cC_setSize, ic, icC, iC;
+	double *pnodeprob;
+	PROB_LIST<double>* pProbList;
+	
+	if(!m_sigmas || !m_betas || nnode < 0 || nnode >= m_numNodes || m_sigmas[nnode] <= 0)
+		return -FLT_MAX;
+	
+	if(!psamples || nsamples < 1)
+		return -FLT_MAX;
+
+	pProbList = (PROB_LIST<double>*)getNodeProb(nnode);
+	pnodeprob = pProbList->pProbs;
+	
+	fconst1 = -0.5 * log(PI2*m_sigmas[nnode]);
+	fconst2 = -0.5 / m_sigmas[nnode];
+		
+	cC_setSize = 0;
+	if(m_numParents[nnode] > 0) {
+		if(findNodeJointProb(nnode, m_parents[nnode], m_numParents[nnode], psamples, nsamples) != ERR_CATNET_OK)
+			return -FLT_MAX;
+		// now m_qcC contains P(x, i, x_j, j\in J_i | y_i, y_j^s, j\in J_i), j = 1,...,numsamples
+		cC_setSize = pcC_size();
+	}
+	else {
+		findNodeMarginalProb(nnode, psamples, nsamples);
+	}
+	
+	fLogLik = 0;
+	if(m_numParents[nnode] > 0) {
+		for(j = 0; j < nsamples; j++) {
+			iC = 0;
+			for(icC = 0; icC < cC_setSize; icC += m_numCategories[nnode]) {
+				for(ic = 0; ic < m_numCategories[nnode]; ic++) {
+					faux = psamples[j * m_numNodes + nnode] - m_betas[nnode][ic];
+					faux = fconst2 * faux * faux;
+					if(pnodeprob[iC*m_numCategories[nnode] + ic] > 0)
+						fLogLik += m_qcC[j*cC_setSize + icC + ic] * (faux + log(pnodeprob[iC*m_numCategories[nnode] + ic]));
+					else
+						fLogLik += m_qcC[j*cC_setSize + icC + ic] * faux;
+				}
+				iC++;
+			}
+		}
+	}
+	else {
+		for(j = 0; j < nsamples; j++) {
+			for(ic = 0; ic < m_numCategories[nnode]; ic++) {
+				faux = psamples[j * m_numNodes + nnode] - m_betas[nnode][ic];
+				faux = fconst2 * faux * faux;
+				if(pnodeprob[ic] > 0)
+					fLogLik += m_qc[j*m_numCategories[nnode] + ic] * (faux + log(pnodeprob[ic]));
+				else
+					fLogLik += m_qc[j*m_numCategories[nnode] + ic] * faux;
+			}
+		}
+	}
+	
+	fLogLik += nsamples*fconst1;
+	//printf("nnode = %d:  fLogLik = %f\n", nnode, fLogLik);
 	return fLogLik;
 }
 
